@@ -758,6 +758,16 @@ mysql_deparse_const(Const *node, deparse_expr_cxt *context)
 		case INTERVALOID:
 			deparse_interval(buf, node->constvalue);
 			break;
+                case BYTEAOID:
+                        /*
+			 * the string for BYTEA always seems to be in the format "\\x##"
+                         * where # is a hex digit, Even if the value passed in is 'hi'::bytea
+                         * we will receive "\x6869". Making this assumption allows us to
+                         * quickly convert postgres escaped strings to mysql ones for comparison
+			 */
+                        extval = OidOutputFunctionCall(typoutput, node->constvalue);
+                        appendStringInfo(buf, "X\'%s\'", extval + 2);
+                        break;			
 		default:
 			extval = OidOutputFunctionCall(typoutput, node->constvalue);
 			mysql_deparse_string_literal(buf, extval);
@@ -1328,8 +1338,17 @@ foreign_expr_walker(Node *node,
 			break;
 		case T_Param:
 			{
-				/* We are not supporting param push down*/
-				return false;
+				Param	   *p = (Param *) node;
+
+				/*
+				 * Collation rule is same as for Consts and non-foreign Vars.
+				 */
+				collation = p->paramcollid;
+				if (collation == InvalidOid ||
+					collation == DEFAULT_COLLATION_OID)
+					state = FDW_COLLATE_NONE;
+				else
+					state = FDW_COLLATE_UNSAFE;
 			}
 			break;
 		case T_ArrayRef:
